@@ -29,12 +29,13 @@ So there are two remaining parties:
 """
 from oauth2.error import OAuthInvalidError, UserNotAuthenticated, \
     AccessTokenNotFound, UserIdentifierMissingError, RedirectUriUnknown, \
-    AuthCodeNotFound
+    AuthCodeNotFound, InvalidSiteAdapter
 from oauth2.compatibility import urlencode, quote
 import json
 import time
 from oauth2.datatype import AuthorizationCode, AccessToken
-from oauth2.web import Response
+from oauth2.web import Response, AuthorizationCodeGrantSiteAdapter, \
+    ImplicitGrantSiteAdapter, ResourceOwnerGrantSiteAdapter
 
 
 def encode_scopes(scopes, use_quote=False):
@@ -52,13 +53,13 @@ def encode_scopes(scopes, use_quote=False):
     return scopes_as_string
 
 
-def json_error_response(error, response):
+def json_error_response(error, response, status_code=400):
     """
     Formats an error as a response containing a JSON body.
     """
     msg = {"error": error.error, "error_description": error.explanation}
 
-    response.status_code = 400
+    response.status_code = status_code
     response.add_header("Content-Type", "application/json")
     response.body = json.dumps(msg)
 
@@ -305,14 +306,17 @@ class AuthorizeMixin(object):
                                     explanation="Authorization denied by user")
 
         try:
-            result = self.site_adapter.authenticate(request, environ, scopes)
+            result = self.site_adapter.authenticate(request, environ, scopes,
+                                                    self.client)
 
-            return self._sanitize_return_value(result)
+            return self.sanitize_return_value(result)
         except UserNotAuthenticated:
             return self.site_adapter.render_auth_page(request, response,
-                                                      environ, scopes)
+                                                      environ, scopes,
+                                                      self.client)
 
-    def _sanitize_return_value(self, value):
+    @staticmethod
+    def sanitize_return_value(value):
         if isinstance(value, tuple) and len(value) is 2:
             return value
 
@@ -379,6 +383,27 @@ class AccessTokenMixin(object):
         return token_data
 
 
+class SiteAdapterMixin(object):
+    """
+    Mixed in by Grant classes that require a site adapter.
+
+    A concrete class must set the class attribute ``site_adapter_class`` that
+    contains a reference to the site adapter class that this class expects.
+    """
+    site_adapter_class = None
+
+    def __init__(self, site_adapter, **kwargs):
+        if isinstance(site_adapter, self.site_adapter_class) is False:
+            raise InvalidSiteAdapter(
+                "Site adapter must inherit from class '{0}'"
+                .format(self.site_adapter_class.__name__)
+            )
+
+        self.site_adapter = site_adapter
+
+        super(SiteAdapterMixin, self).__init__(**kwargs)
+
+
 class AuthorizationCodeAuthHandler(AuthorizeMixin, AuthRequestMixin,
                                    GrantHandler):
     """
@@ -443,7 +468,7 @@ class AuthorizationCodeAuthHandler(AuthorizeMixin, AuthRequestMixin,
         query = "code=" + code
 
         if self.state is not None:
-            query += "&state=" + self.state
+            query += "&state=" + quote(self.state)
 
         return "%s?%s" % (self.client.redirect_uri, query)
 
@@ -556,7 +581,8 @@ class AuthorizationCodeTokenHandler(AccessTokenMixin, GrantHandler):
         self.user_id = stored_code.user_id
 
 
-class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
+class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant,
+                             SiteAdapterMixin):
     """
     Implementation of the Authorization Code Grant auth flow.
 
@@ -568,10 +594,15 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
         auth_controller = AuthorizationController()
 
         auth_controller.add_grant_type(AuthorizationCodeGrant())
+
+    .. versionchanged:: 1.0.0
+       Require parameter ``site_adapter``.
     """
 
     grant_type = "authorization_code"
 
+    site_adapter_class = AuthorizationCodeGrantSiteAdapter
+
     def __init__(self, unique_token=False, expires_in=0, **kwargs):
         self.unique_token = unique_token
         self.expires_in = expires_in
@@ -579,7 +610,18 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
         super(AuthorizationCodeGrant, self).__init__(**kwargs)
 
     def __call__(self, request, server):
-        if (request.post_param("grant_type") == "authorization_code"
+        """
+        :param request: Incoming request
+        :type request: oauth2.web.Request
+
+        :param server: The OAuth2 provider instance
+
+
+        .. versionchanged:: 1.0.0
+           Check the HTTP method of a request
+        """
+        if (request.method == "POST"
+                and request.post_param("grant_type") == "authorization_code"
                 and request.path == server.token_path):
             return AuthorizationCodeTokenHandler(
                 access_token_store=server.access_token_store,
@@ -588,7 +630,8 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
                 token_generator=server.token_generator,
                 unique_token=self.unique_token)
 
-        if (request.get_param("response_type") == "code"
+        if (request.method == "GET"
+                and request.get_param("response_type") == "code"
                 and request.path == server.authorize_path):
             scope_handler = self._create_scope_handler()
 
@@ -596,13 +639,13 @@ class AuthorizationCodeGrant(GrantHandlerFactory, ScopeGrant):
                 auth_token_store=server.auth_code_store,
                 client_authenticator=server.client_authenticator,
                 scope_handler=scope_handler,
-                site_adapter=server.site_adapter,
+                site_adapter=self.site_adapter,
                 token_generator=server.token_generator)
 
         return None
 
 
-class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
+class ImplicitGrant(GrantHandlerFactory, ScopeGrant, SiteAdapterMixin):
     """
     Implementation of the Implicit Grant auth flow.
 
@@ -614,10 +657,15 @@ class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
         auth_controller = AuthorizationController()
 
         auth_controller.add_grant_type(ImplicitGrant())
+
+    .. versionchanged:: 1.0.0
+       Require parameter ``site_adapter``.
     """
 
     grant_type = "implicit"
 
+    site_adapter_class = ImplicitGrantSiteAdapter
+
     def __call__(self, request, server):
         response_type = request.get_param("response_type")
 
@@ -627,7 +675,7 @@ class ImplicitGrant(GrantHandlerFactory, ScopeGrant):
                 access_token_store=server.access_token_store,
                 client_authenticator=server.client_authenticator,
                 scope_handler=self._create_scope_handler(),
-                site_adapter=server.site_adapter,
+                site_adapter=self.site_adapter,
                 token_generator=server.token_generator)
         return None
 
@@ -671,7 +719,7 @@ class ImplicitGrantHandler(AuthorizeMixin, AuthRequestMixin, GrantHandler):
             format(self.client.redirect_uri, token)
 
         if self.state is not None:
-            uri_with_fragment += "&state=" + self.state
+            uri_with_fragment += "&state=" + quote(self.state)
 
         if self.scope_handler.send_back is True:
             scopes_as_string = encode_scopes(self.scope_handler.scopes,
@@ -685,7 +733,7 @@ class ImplicitGrantHandler(AuthorizeMixin, AuthRequestMixin, GrantHandler):
         return response
 
 
-class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant):
+class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant, SiteAdapterMixin):
     """
     Implementation of the Resource Owner Password Credentials Grant auth flow.
 
@@ -699,10 +747,15 @@ class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant):
         auth_controller = AuthorizationController()
 
         auth_controller.add_grant_type(ResourceOwnerGrant())
+
+    .. versionchanged:: 1.0.0
+       Require parameter ``site_adapter``.
     """
 
     grant_type = "password"
 
+    site_adapter_class = ResourceOwnerGrantSiteAdapter
+
     def __init__(self, unique_token=False, expires_in=0, **kwargs):
         self.unique_token = unique_token
         self.expires_in = expires_in
@@ -721,22 +774,35 @@ class ResourceOwnerGrant(GrantHandlerFactory, ScopeGrant):
             access_token_store=server.access_token_store,
             client_authenticator=server.client_authenticator,
             scope_handler=self._create_scope_handler(),
-            site_adapter=server.site_adapter,
+            site_adapter=self.site_adapter,
             token_generator=server.token_generator,
             unique_token=self.unique_token)
 
 
-class ResourceOwnerGrantHandler(GrantHandler, AuthorizeMixin,
-                                AccessTokenMixin):
+class ResourceOwnerGrantHandler(GrantHandler, AccessTokenMixin):
     """
     Class for handling Resource Owner authorization requests.
 
     See http://tools.ietf.org/html/rfc6749#section-4.3
     """
 
-    def __init__(self, client_authenticator, scope_handler, **kwargs):
+    OWNER_NOT_AUTHENTICATED = "Unable to authenticate resource owner"
+
+    def __init__(self, client_authenticator, scope_handler, site_adapter,
+                 **kwargs):
+        """
+        :param client_authenticator: Client authenticator
+        :type client_authenticator: oauth2.client_authenticator.ClientAuthenticator
+
+        :param scope_handler: Scope handler
+        :type scope_handler: oauth2.grant.Scope
+
+        :param site_adapter: Site adapter
+        :type site_adapter: oauth2.web.SiteAdapter
+        """
         self.client_authenticator = client_authenticator
         self.scope_handler = scope_handler
+        self.site_adapter = site_adapter
 
         self.client = None
         self.password = None
@@ -750,9 +816,14 @@ class ResourceOwnerGrantHandler(GrantHandler, AuthorizeMixin,
         it and issues a new access token that is returned to the client on
         successful validation.
         """
-        data = self.authorize(request=request, response=response,
-                              environ=environ,
-                              scopes=self.scope_handler.scopes)
+        try:
+            data = self.site_adapter.authenticate(request, environ,
+                                                  self.scope_handler.scopes,
+                                                  self.client)
+            data = AuthorizeMixin.sanitize_return_value(data)
+        except UserNotAuthenticated:
+            raise OAuthInvalidError(error="invalid_client",
+                                    explanation=self.OWNER_NOT_AUTHENTICATED)
 
         if isinstance(data, Response):
             return data
@@ -785,7 +856,11 @@ class ResourceOwnerGrantHandler(GrantHandler, AuthorizeMixin,
         return True
 
     def handle_error(self, error, response):
-        return json_error_response(error, response)
+        status_code = 400
+        if error.explanation == self.OWNER_NOT_AUTHENTICATED:
+            status_code = 401
+
+        return json_error_response(error, response, status_code=status_code)
 
 
 class RefreshToken(GrantHandlerFactory, ScopeGrant):
@@ -809,8 +884,7 @@ class RefreshToken(GrantHandlerFactory, ScopeGrant):
 
     grant_type = "refresh_token"
 
-    def __init__(self, expires_in,
-                 reissue_refresh_tokens=False, **kwargs):
+    def __init__(self, expires_in, reissue_refresh_tokens=False, **kwargs):
 
         self.refresh_expires_in = expires_in
         self.reissue_refresh_tokens = reissue_refresh_tokens
@@ -834,8 +908,8 @@ class RefreshToken(GrantHandlerFactory, ScopeGrant):
             client_authenticator=server.client_authenticator,
             scope_handler=self._create_scope_handler(),
             token_generator=server.token_generator,
-            reissue_refresh_tokens = self.reissue_refresh_tokens,
-            )
+            reissue_refresh_tokens=self.reissue_refresh_tokens
+        )
 
 
 class RefreshTokenHandler(GrantHandler):
@@ -853,6 +927,7 @@ class RefreshTokenHandler(GrantHandler):
 
         self.client = None
         self.data = {}
+        self.refresh_grant_type = None
         self.refresh_token = None
         self.user_id = None
 
@@ -923,7 +998,6 @@ class RefreshTokenHandler(GrantHandler):
             raise OAuthInvalidError(error="invalid_request",
                                     explanation="Invalid refresh token")
 
-
         refresh_token_expires_at = access_token.refresh_expires_at
         self.refresh_grant_type = access_token.grant_type
 
