@@ -1,13 +1,16 @@
 from mock import Mock, call, patch
 import json
 from oauth2.client_authenticator import ClientAuthenticator
+from oauth2.compatibility import quote
 from oauth2.test import unittest
-from oauth2.web import Request, Response, SiteAdapter
+from oauth2.web import Response, ResourceOwnerGrantSiteAdapter, \
+    ImplicitGrantSiteAdapter, AuthorizationCodeGrantSiteAdapter
+from oauth2.web.wsgi import Request
 from oauth2.grant import ImplicitGrantHandler, AuthorizationCodeAuthHandler, \
     AuthRequestMixin, AuthorizationCodeTokenHandler, ImplicitGrant, \
     AuthorizationCodeGrant, ResourceOwnerGrantHandler, ResourceOwnerGrant, \
     Scope, RefreshToken, RefreshTokenHandler, ClientCredentialsGrant, \
-    ClientCredentialsHandler, AuthorizeMixin
+    ClientCredentialsHandler, AuthorizeMixin, SiteAdapterMixin
 from oauth2.store import ClientStore, AuthCodeStore, AccessTokenStore
 from oauth2.error import OAuthInvalidError, UserNotAuthenticated, \
     AccessTokenNotFound, UserIdentifierMissingError, AuthCodeNotFound
@@ -27,10 +30,12 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         AuthorizationCodeGrant() should return a new instance of AuthorizationCodeAuthHandler on request
         """
         default_scope = "default_scope"
+        method = "GET"
         scopes = ["first", "second"]
         path = "/auth"
 
         request_mock = Mock(spec=Request)
+        request_mock.method = method
         request_mock.path = path
         request_mock.get_param.return_value = "code"
 
@@ -40,12 +45,14 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         server_mock.authorize_path = path
         server_mock.auth_code_store = Mock()
         server_mock.client_authenticator = Mock()
-        server_mock.site_adapter = Mock()
         server_mock.token_generator = Mock()
 
-        factory = AuthorizationCodeGrant(default_scope=default_scope,
-                                         scopes=scopes,
-                                         scope_class=scope_mock)
+        factory = AuthorizationCodeGrant(
+            default_scope=default_scope,
+            scopes=scopes,
+            scope_class=scope_mock,
+            site_adapter=Mock(spec=AuthorizationCodeGrantSiteAdapter)
+        )
         result_class = factory(request_mock, server_mock)
 
         request_mock.get_param.assert_called_with("response_type")
@@ -56,6 +63,7 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         path = "/token"
 
         request_mock = Mock(spec=Request)
+        request_mock.method = "POST"
         request_mock.path = path
         request_mock.post_param.return_value = "authorization_code"
 
@@ -67,7 +75,9 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
         server_mock.client_store = Mock()
         server_mock.token_generator = Mock()
 
-        factory = AuthorizationCodeGrant()
+        factory = AuthorizationCodeGrant(
+            site_adapter=Mock(spec=AuthorizationCodeGrantSiteAdapter)
+        )
         result_class = factory(request_mock, server_mock)
 
         request_mock.post_param.assert_called_with("grant_type")
@@ -76,13 +86,15 @@ class AuthorizationCodeGrantTestCase(unittest.TestCase):
 
     def test_create_no_match(self):
         request_mock = Mock(spec=Request)
+        request_mock.method = "POST"
         request_mock.get_param.return_value = "no-code"
         request_mock.post_param.return_value = "no-authorization_code"
 
-        factory = AuthorizationCodeGrant()
+        factory = AuthorizationCodeGrant(
+            site_adapter=Mock(spec=AuthorizationCodeGrantSiteAdapter)
+        )
         result_class = factory(request_mock, Mock())
 
-        request_mock.get_param.assert_called_with("response_type")
         request_mock.post_param.assert_called_with("grant_type")
         self.assertEqual(result_class, None)
 
@@ -125,7 +137,7 @@ class AuthorizeMixinTestCase(unittest.TestCase):
         """
         AuthorizeMixin.authorize should raise an OAuthUserError if the user did not authorize the request
         """
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.user_has_denied_access.return_value = True
 
         auth_mixin = AuthorizeMixin(site_adapter=site_adapter_mock)
@@ -137,13 +149,16 @@ class AuthorizeMixinTestCase(unittest.TestCase):
         """
         AuthorizeMixin.authorize should return a tuple even if the SiteAdapter returns a dict
         """
+        client_mock = Mock(spec=Client)
+
         test_data = {"test": "data"}
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.user_has_denied_access.return_value = False
         site_adapter_mock.authenticate.return_value = test_data
 
         auth_mixin = AuthorizeMixin(site_adapter=site_adapter_mock)
+        auth_mixin.client = client_mock
         result = auth_mixin.authorize(Mock(spec=Request), Mock(spec=Response),
                                       environ={}, scopes=[])
 
@@ -155,13 +170,16 @@ class AuthorizeMixinTestCase(unittest.TestCase):
         """
         AuthorizeMixin.authorize should return the tuple returned by the SiteAdapter
         """
+        client_mock = Mock(spec=Client)
+
         test_data = ({"test": "data"}, 123)
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.user_has_denied_access.return_value = False
         site_adapter_mock.authenticate.return_value = test_data
 
         auth_mixin = AuthorizeMixin(site_adapter=site_adapter_mock)
+        auth_mixin.client = client_mock
         result = auth_mixin.authorize(Mock(spec=Request), Mock(spec=Response),
                                       environ={}, scopes=[])
 
@@ -170,14 +188,17 @@ class AuthorizeMixinTestCase(unittest.TestCase):
         self.assertEqual(result[1], test_data[1])
 
     def test_authorize_user_not_authenticated(self):
+        client_mock = Mock(spec=Client)
+
         response_mock = Mock(spec=Response)
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.user_has_denied_access.return_value = False
         site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
         site_adapter_mock.render_auth_page.return_value = response_mock
 
         auth_mixin = AuthorizeMixin(site_adapter=site_adapter_mock)
+        auth_mixin.client = client_mock
         result = auth_mixin.authorize(Mock(spec=Request), response_mock,
                                       environ={}, scopes=[])
 
@@ -189,14 +210,17 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         code = "abcd"
         environ = {"session": "data"}
         scopes = ["scope"]
-        state = "mystate"
+        state = "my%state"
         redirect_uri = "https://callback"
         user_data = {"user_id": 789}
 
-        location_uri = "%s?code=%s&state=%s" % (redirect_uri, code, state)
+        location_uri = "%s?code=%s&state=%s" % (redirect_uri, code, quote(state))
 
         auth_code_store_mock = Mock(spec=AuthCodeStore)
 
+        client = Client(identifier="abc", secret="xyz",
+                        redirect_uris=[redirect_uri])
+
         response_mock = Mock(spec=Response)
 
         request_mock = Mock(spec=Request)
@@ -205,7 +229,7 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = False
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=AuthorizationCodeGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = user_data
         site_adapter_mock.user_has_denied_access.return_value = False
 
@@ -219,14 +243,14 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
             token_generator=token_generator_mock
         )
 
-        handler.client = Client(identifier="abc", secret="xyz",
-                                redirect_uris=[redirect_uri])
+        handler.client = client
         handler.state = state
         response = handler.process(request_mock, response_mock, environ)
 
         token_generator_mock.generate.assert_called_with()
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ, scopes)
+                                                          environ, scopes,
+                                                          client)
         self.assertTrue(auth_code_store_mock.save_code.called)
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response.body, "")
@@ -240,12 +264,14 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
         response_mock = Mock(spec=Response)
         scopes = ["scopes"]
 
+        client_mock = Mock(spec=Client)
+
         request_mock = Mock(spec=Request)
 
         scope_handler_mock = Mock(Scope)
         scope_handler_mock.scopes = scopes
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=AuthorizationCodeGrantSiteAdapter)
         site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
         site_adapter_mock.render_auth_page.return_value = response_mock
 
@@ -254,12 +280,14 @@ class AuthorizationCodeAuthHandlerTestCase(unittest.TestCase):
             scope_handler=scope_handler_mock, site_adapter=site_adapter_mock,
             token_generator=Mock()
         )
+        handler.client = client_mock
         response = handler.process(request_mock, response_mock, environ)
 
         site_adapter_mock.render_auth_page.assert_called_with(request_mock,
                                                               response_mock,
                                                               environ,
-                                                              scopes)
+                                                              scopes,
+                                                              client_mock)
         self.assertEqual(response, response_mock)
 
     def test_redirect_oauth_error(self):
@@ -761,16 +789,14 @@ class ImplicitGrantTestCase(unittest.TestCase):
         request_mock.path = path
         request_mock.get_param.return_value = "token"
 
-        site_adapter_mock = Mock()
         token_generator_mock = Mock()
 
         server_mock = Mock()
         server_mock.authorize_path = path
         server_mock.client_authenticator = Mock()
-        server_mock.site_adapter = site_adapter_mock
         server_mock.token_generator = token_generator_mock
 
-        factory = ImplicitGrant()
+        factory = ImplicitGrant(site_adapter=ImplicitGrantSiteAdapter())
         result_class = factory(request_mock, server_mock)
 
         request_mock.get_param.assert_called_with("response_type")
@@ -782,15 +808,15 @@ class ImplicitGrantTestCase(unittest.TestCase):
 
         server_mock = Mock()
 
-        factory = ImplicitGrant()
+        factory = ImplicitGrant(site_adapter=ImplicitGrantSiteAdapter())
         result_class = factory(request_mock, server_mock)
 
         request_mock.get_param.assert_called_with("response_type")
         self.assertEqual(result_class, None)
 
+
 class ImplicitGrantHandlerTestCase(unittest.TestCase):
     def test_process_redirect_with_token(self):
-        client_id = "abc"
         environ = {"session": "data"}
         redirect_uri = "http://callback"
         scopes = ["scopes"]
@@ -799,6 +825,9 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
 
         access_token_store_mock = Mock(spec=AccessTokenStore)
 
+        client = Client(identifier="abc", secret="xyz",
+                        redirect_uris=[redirect_uri])
+
         request_mock = Mock(spec=Request)
 
         responseMock = Mock(spec=Response)
@@ -807,7 +836,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = False
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = user_data
 
         token_generator_mock = Mock(spec=["generate"])
@@ -820,12 +849,12 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
             client_authenticator=Mock(), scope_handler=scope_handler_mock,
             site_adapter=site_adapter_mock,
             token_generator=token_generator_mock)
-        handler.client = Client(identifier="abc", secret="xyz",
-                                redirect_uris=[redirect_uri])
+        handler.client = client
         result_response = handler.process(request_mock, responseMock, environ)
 
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ, scopes)
+                                                          environ, scopes,
+                                                          client)
 
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
@@ -843,11 +872,11 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         ImplicitGrantHandler should include the value of the "state" query parameter from request in redirect
         """
         redirect_uri = "http://callback"
-        state = "XHGFI"
+        state = "XH%GFI"
         token = "tokencode"
         user_data = ({}, 1)
 
-        expected_redirect_uri = "%s#access_token=%s&token_type=bearer&state=%s" % (redirect_uri, token, state)
+        expected_redirect_uri = "%s#access_token=%s&token_type=bearer&state=%s" % (redirect_uri, token, quote(state))
 
         response_mock = Mock(spec=Response)
 
@@ -855,7 +884,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = []
         scope_handler_mock.send_back = False
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = user_data
 
         token_generator_mock = Mock(spec=["generate"])
@@ -895,7 +924,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = True
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = ({}, 1)
 
         token_generator_mock = Mock(spec=["generate"])
@@ -923,6 +952,8 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scopes = ["scopes"]
         environ = {"session": "data"}
 
+        client_mock = Mock(spec=Client)
+
         request_mock = Mock(spec=Request)
 
         response_mock = Mock(spec=Response)
@@ -930,7 +961,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock = Mock(Scope)
         scope_handler_mock.scopes = scopes
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
         site_adapter_mock.render_auth_page.return_value = response_mock
 
@@ -939,14 +970,17 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
             scope_handler=scope_handler_mock, site_adapter=site_adapter_mock,
             token_generator=Mock()
         )
+        handler.client = client_mock
         result_response = handler.process(request_mock, response_mock, environ)
 
         site_adapter_mock.authenticate.assert_called_with(request_mock,
-                                                          environ, scopes)
+                                                          environ, scopes,
+                                                          client_mock)
         site_adapter_mock.render_auth_page.assert_called_with(request_mock,
                                                               response_mock,
                                                               environ,
-                                                              scopes)
+                                                              scopes,
+                                                              client_mock)
         self.assertEqual(result_response, response_mock)
 
     def test_process_user_denied_access(self):
@@ -957,7 +991,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock = Mock(spec=Scope)
         scope_handler_mock.scopes = []
 
-        site_adapter_mock = Mock(spec=SiteAdapter)
+        site_adapter_mock = Mock(spec=ImplicitGrantSiteAdapter)
         site_adapter_mock.user_has_denied_access.return_value = True
 
         handler = ImplicitGrantHandler(
@@ -996,7 +1030,7 @@ class ImplicitGrantHandlerTestCase(unittest.TestCase):
         handler.client = Client(identifier="abc", secret="xyz",
                                 redirect_uris=[redirect_uri])
         altered_response = handler.handle_error(error_mock,
-                                                        response_mock)
+                                                response_mock)
 
         response_mock.add_header.assert_called_with(
             "Location",
@@ -1011,16 +1045,16 @@ class ResourceOwnerGrantTestCase(unittest.TestCase):
         request_mock.post_param.return_value = "password"
 
         access_token_store_mock = Mock(AccessTokenStore)
-        site_adapter_mock = Mock(SiteAdapter)
         token_generator_mock = Mock()
 
         server_mock = Mock(Provider)
         server_mock.access_token_store = access_token_store_mock
         server_mock.client_authenticator = Mock(ClientAuthenticator)
-        server_mock.site_adapter = site_adapter_mock
         server_mock.token_generator = token_generator_mock
 
-        factory = ResourceOwnerGrant()
+        factory = ResourceOwnerGrant(
+            site_adapter=ResourceOwnerGrantSiteAdapter()
+        )
 
         handler = factory(request_mock, server_mock)
 
@@ -1033,16 +1067,20 @@ class ResourceOwnerGrantTestCase(unittest.TestCase):
 
         server_mock = Mock(Provider)
 
-        factory = ResourceOwnerGrant()
+        factory = ResourceOwnerGrant(
+            site_adapter=ResourceOwnerGrantSiteAdapter()
+        )
 
         handler = factory(request_mock, server_mock)
 
         request_mock.post_param.assert_called_with("grant_type")
         self.assertEqual(handler, None)
 
+
 class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
     def test_process(self):
         access_token = "0aef"
+        client = Client(identifier="abc", secret="xyz")
         expected_response_body = {"access_token": access_token,
                                   "token_type": "Bearer"}
         scopes = ["scope"]
@@ -1059,7 +1097,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = False
 
-        site_adapter_mock = Mock(SiteAdapter)
+        site_adapter_mock = Mock(spec=ResourceOwnerGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = user
 
         token_generator_mock = Mock(spec=TokenGenerator)
@@ -1071,11 +1109,11 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
             scope_handler=scope_handler_mock,
             site_adapter=site_adapter_mock,
             token_generator=token_generator_mock)
-        handler.client = Client(identifier="abc", secret="xyz")
+        handler.client = client
         result = handler.process(request_mock, response_mock, {})
 
         site_adapter_mock.authenticate.assert_called_with(request_mock, {},
-                                                          scopes)
+                                                          scopes, client)
         token_generator_mock.create_access_token_data.assert_called_with(ResourceOwnerGrant.grant_type)
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
@@ -1093,7 +1131,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
     @patch("time.time", mock_time)
     def test_process_with_refresh_token(self):
         access_token = "0aef"
-        client_id = "abcd"
+        client = Client(identifier="abc", secret="xyz")
         expected_response_body = {"access_token": access_token,
                                   "token_type": "Bearer",
                                   "refresh_token": "wxyz", "expires_in": 600}
@@ -1112,7 +1150,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         scope_handler_mock.scopes = scopes
         scope_handler_mock.send_back = False
 
-        site_adapter_mock = Mock(SiteAdapter)
+        site_adapter_mock = Mock(spec=ResourceOwnerGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = user
 
         token_generator_mock = Mock(spec=TokenGenerator)
@@ -1125,11 +1163,11 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
             scope_handler=scope_handler_mock,
             site_adapter=site_adapter_mock,
             token_generator=token_generator_mock)
-        handler.client = Client(identifier="abc", secret="xyz")
+        handler.client = client
         result = handler.process(request_mock, response_mock, {})
 
         site_adapter_mock.authenticate.assert_called_with(request_mock, {},
-                                                          scopes)
+                                                          scopes, client)
         token_generator_mock.create_access_token_data.assert_called_with(ResourceOwnerGrant.grant_type)
         access_token, = access_token_store_mock.save_token.call_args[0]
         self.assertTrue(isinstance(access_token, AccessToken))
@@ -1155,7 +1193,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
 
         response_mock = Mock(Response)
 
-        site_adapter_mock = Mock(SiteAdapter)
+        site_adapter_mock = Mock(spec=ResourceOwnerGrantSiteAdapter)
         site_adapter_mock.authenticate.return_value = ({"test": "data"}, 123)
 
         scope_handler_mock = Mock(Scope)
@@ -1184,9 +1222,35 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         self.assertDictEqual(expected_response_body, json.loads(result.body))
         self.assertEqual(result, response_mock)
 
+    def test_process_invalid_user(self):
+        response_mock = Mock(Response)
+
+        scope_handler_mock = Mock(Scope)
+        scope_handler_mock.scopes = ["scopes"]
+        scope_handler_mock.send_back = False
+
+        site_adapter_mock = Mock(spec=ResourceOwnerGrantSiteAdapter)
+
+        site_adapter_mock.authenticate.side_effect = UserNotAuthenticated
+
+        handler = ResourceOwnerGrantHandler(
+            access_token_store=Mock(AccessTokenStore),
+            client_authenticator=Mock(ClientAuthenticator),
+            scope_handler=scope_handler_mock,
+            site_adapter=site_adapter_mock,
+            token_generator=Mock(TokenGenerator))
+        handler.client = Client(identifier="abc", secret="xyz")
+
+        with self.assertRaises(OAuthInvalidError) as expected:
+            handler.process(Mock(Request), response_mock, {})
+
+        e = expected.exception
+
+        self.assertEqual(e.error, "invalid_client")
+        self.assertEqual(e.explanation,
+                         ResourceOwnerGrantHandler.OWNER_NOT_AUTHENTICATED)
+
     def test_read_validate_params(self):
-        client_id = "abcd"
-        client_secret = "xyz"
         password = "johnpw"
         username = "johndoe"
 
@@ -1204,7 +1268,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
             access_token_store=Mock(AccessTokenStore),
             client_authenticator=client_auth_mock,
             scope_handler=scope_handler_mock,
-            site_adapter=Mock(SiteAdapter),
+            site_adapter=Mock(spec=ResourceOwnerGrantSiteAdapter),
             token_generator=Mock())
         result = handler.read_validate_params(request_mock)
 
@@ -1217,6 +1281,24 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         self.assertEqual(handler.password, password)
         self.assertTrue(result)
 
+    def test_handle_error_owner_not_authenticated(self):
+        error = OAuthInvalidError(
+            error="invalid_client",
+            explanation=ResourceOwnerGrantHandler.OWNER_NOT_AUTHENTICATED)
+
+        response = Response()
+
+        handler = ResourceOwnerGrantHandler(
+            access_token_store=Mock(AccessTokenStore),
+            client_authenticator=Mock(),
+            scope_handler=Mock(),
+            site_adapter=Mock(),
+            token_generator=Mock())
+
+        result = handler.handle_error(error, response)
+
+        self.assertEqual(result.status_code, 401)
+
 
 class ScopeTestCase(unittest.TestCase):
     def test_parse_scope_scope_present_in_query(self):
